@@ -14,7 +14,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 
 #include <dbus/dbus.h>
@@ -54,6 +54,11 @@ typedef struct _FcitxDBusPropertyTable {
     void (*setfunc)(void* arg, DBusMessageIter* iter);
 } FcitxDBusPropertyTable;
 
+typedef struct _FcitxIPCKeyEvent {
+    FcitxKeySym sym;
+    unsigned int state;
+} FcitxIPCKeyEvent;
+
 static void* IPCCreate(FcitxInstance* instance, int frontendid);
 static boolean IPCDestroy(void* arg);
 void IPCCreateIC(void* arg, FcitxInputContext* context, void *priv);
@@ -135,6 +140,8 @@ const char * im_introspection_xml =
     "      <arg name=\"keyval2\" direction=\"out\" type=\"u\"/>\n"
     "      <arg name=\"state2\" direction=\"out\" type=\"u\"/>\n"
     "    </method>\n"
+    "    <method name=\"Exit\">\n"
+    "    </method>\n"
     "    <property access=\"readwrite\" type=\"a(sssb)\" name=\"IMList\">\n"
     "      <annotation name=\"org.freedesktop.DBus.Property.EmitsChangedSignal\" value=\"true\"/>"
     "    </property>\n"
@@ -230,7 +237,7 @@ int ABI_VERSION = FCITX_ABI_VERSION;
 
 void* IPCCreate(FcitxInstance* instance, int frontendid)
 {
-    FcitxIPCFrontend* ipc = fcitx_malloc0(sizeof(FcitxIPCFrontend));
+    FcitxIPCFrontend* ipc = fcitx_utils_malloc0(sizeof(FcitxIPCFrontend));
     ipc->frontendid = frontendid;
     ipc->owner = instance;
 
@@ -251,11 +258,11 @@ void* IPCCreate(FcitxInstance* instance, int frontendid)
         free(ipc);
         return NULL;
     }
-    
+
     FcitxIMEventHook hook;
     hook.arg = ipc;
     hook.func = IPCUpdateIMList;
-    RegisterUpdateIMListHook(instance, hook);
+    FcitxInstanceRegisterUpdateIMListHook(instance, hook);
 
     return ipc;
 }
@@ -268,10 +275,10 @@ boolean IPCDestroy(void* arg)
 void IPCCreateIC(void* arg, FcitxInputContext* context, void* priv)
 {
     FcitxIPCFrontend* ipc = (FcitxIPCFrontend*) arg;
-    FcitxIPCIC* ipcic = (FcitxIPCIC*) fcitx_malloc0(sizeof(FcitxIPCIC));
+    FcitxIPCIC* ipcic = (FcitxIPCIC*) fcitx_utils_malloc0(sizeof(FcitxIPCIC));
     DBusMessage* message = (DBusMessage*) priv;
     DBusMessage *reply = dbus_message_new_method_return(message);
-    FcitxConfig* config = FcitxInstanceGetConfig(ipc->owner);
+    FcitxGlobalConfig* config = FcitxInstanceGetGlobalConfig(ipc->owner);
 
     context->privateic = ipcic;
 
@@ -287,6 +294,7 @@ void IPCCreateIC(void* arg, FcitxInputContext* context, void* priv)
     if (dbus_message_is_method_call(message, FCITX_IM_DBUS_INTERFACE, "CreateIC")) {
         /* CreateIC v1 indicates that default state can only be disabled */
         context->state = IS_CLOSED;
+        context->contextCaps |= CAPACITY_CLIENT_SIDE_CONTROL_STATE;
         ipcic->appname = NULL;
         dbus_message_append_args(reply,
                                  DBUS_TYPE_INT32, &ipcic->id,
@@ -310,7 +318,7 @@ void IPCCreateIC(void* arg, FcitxInputContext* context, void* priv)
         }
 
         if (config->shareState == ShareState_PerProgram)
-            SetICStateFromSameApplication(ipc->owner, ipc->frontendid, context);
+            FcitxInstanceSetICStateFromSameApplication(ipc->owner, ipc->frontendid, context);
 
         boolean arg0 = context->state != IS_CLOSED;
 
@@ -456,10 +464,17 @@ static DBusHandlerResult IPCDBusEventHandler(DBusConnection *connection, DBusMes
         FcitxDBusPropertyGetAll(ipc, msg);
         return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "CreateIC")) {
-        CreateIC(ipc->owner, ipc->frontendid, msg);
+        FcitxInstanceCreateIC(ipc->owner, ipc->frontendid, msg);
         return DBUS_HANDLER_RESULT_HANDLED;
     } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "CreateICv2")) {
-        CreateIC(ipc->owner, ipc->frontendid, msg);
+        FcitxInstanceCreateIC(ipc->owner, ipc->frontendid, msg);
+        return DBUS_HANDLER_RESULT_HANDLED;
+    } else if (dbus_message_is_method_call(msg, FCITX_IM_DBUS_INTERFACE, "Exit")) {
+        DBusMessage *reply = dbus_message_new_method_return(msg);
+        dbus_connection_send(ipc->conn, reply, NULL);
+        dbus_message_unref(reply);
+        dbus_connection_flush(ipc->conn);
+        FcitxInstanceEnd(ipc->owner);
         return DBUS_HANDLER_RESULT_HANDLED;
     }
     return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
@@ -471,7 +486,7 @@ static DBusHandlerResult IPCICDBusEventHandler(DBusConnection *connection, DBusM
     FcitxIPCFrontend* ipc = (FcitxIPCFrontend*) user_data;
     int id;
     sscanf(dbus_message_get_path(msg), FCITX_IC_DBUS_PATH, &id);
-    FcitxInputContext* ic = FindIC(ipc->owner, ipc->frontendid, &id);
+    FcitxInputContext* ic = FcitxInstanceFindIC(ipc->owner, ipc->frontendid, &id);
     DBusHandlerResult result = DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 
     if (dbus_message_is_method_call(msg, DBUS_INTERFACE_INTROSPECTABLE, "Introspect")) {
@@ -487,13 +502,13 @@ static DBusHandlerResult IPCICDBusEventHandler(DBusConnection *connection, DBusM
         DBusError error;
         dbus_error_init(&error);
         if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "EnableIC")) {
-            EnableIM(ipc->owner, ic, false);
+            FcitxInstanceEnableIM(ipc->owner, ic, false);
             DBusMessage *reply = dbus_message_new_method_return(msg);
             dbus_connection_send(ipc->conn, reply, NULL);
             dbus_message_unref(reply);
             result = DBUS_HANDLER_RESULT_HANDLED;
         } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "CloseIC")) {
-            CloseIM(ipc->owner, ic);
+            FcitxInstanceCloseIM(ipc->owner, ic);
             DBusMessage *reply = dbus_message_new_method_return(msg);
             dbus_connection_send(ipc->conn, reply, NULL);
             dbus_message_unref(reply);
@@ -534,8 +549,8 @@ static DBusHandlerResult IPCICDBusEventHandler(DBusConnection *connection, DBusM
                 dbus_message_unref(reply);
             }
             result = DBUS_HANDLER_RESULT_HANDLED;
-        } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "DestroyIC")) {
-            DestroyIC(ipc->owner, ipc->frontendid, &id);
+        } else if (dbus_message_is_method_call(msg, FCITX_IC_DBUS_INTERFACE, "FcitxInstanceDestroyIC")) {
+            FcitxInstanceDestroyIC(ipc->owner, ipc->frontendid, &id);
             DBusMessage *reply = dbus_message_new_method_return(msg);
             dbus_connection_send(ipc->conn, reply, NULL);
             dbus_message_unref(reply);
@@ -571,20 +586,20 @@ static DBusHandlerResult IPCICDBusEventHandler(DBusConnection *connection, DBusM
 
 static int IPCProcessKey(FcitxIPCFrontend* ipc, FcitxInputContext* callic, uint32_t originsym, uint32_t keycode, uint32_t originstate, uint32_t t, FcitxKeyEventType type)
 {
-    FcitxInputContext* ic = GetCurrentIC(ipc->owner);
-    FcitxConfig* config = FcitxInstanceGetConfig(ipc->owner);
+    FcitxInputContext* ic = FcitxInstanceGetCurrentIC(ipc->owner);
+    FcitxGlobalConfig* config = FcitxInstanceGetGlobalConfig(ipc->owner);
     FcitxInputState* input = FcitxInstanceGetInputState(ipc->owner);
 
     if (ic == NULL) {
-        SetCurrentIC(ipc->owner, callic);
+        FcitxInstanceSetCurrentIC(ipc->owner, callic);
     }
     ic = callic;
     FcitxKeySym sym;
     unsigned int state;
 
-    originstate = originstate - (originstate & KEY_NUMLOCK) - (originstate & KEY_CAPSLOCK) - (originstate & KEY_SCROLLLOCK);
-    originstate &= KEY_USED_MASK;
-    GetKey(originsym, originstate, &sym, &state);
+    originstate = originstate - (originstate & FcitxKeyState_NumLock) - (originstate & FcitxKeyState_CapsLock) - (originstate & FcitxKeyState_ScrollLock);
+    originstate &= FcitxKeyState_UsedMask;
+    FcitxHotkeyGetKey(originsym, originstate, &sym, &state);
     FcitxLog(DEBUG,
              "KeyRelease=%d  state=%d  KEYCODE=%d  KEYSYM=%u ",
              (type == FCITX_RELEASE_KEY), state, keycode, sym);
@@ -592,15 +607,22 @@ static int IPCProcessKey(FcitxIPCFrontend* ipc, FcitxInputContext* callic, uint3
     if (originsym == 0)
         return 0;
 
-    if (ic->state == IS_CLOSED && type == FCITX_PRESS_KEY && IsHotKey(sym, state, config->hkTrigger)) {
-        EnableIM(ipc->owner, ic, false);
+    if (ic->state == IS_CLOSED && type == FCITX_PRESS_KEY && FcitxHotkeyIsHotKey(sym, state, config->hkTrigger)) {
+        FcitxInstanceEnableIM(ipc->owner, ic, false);
         FcitxInputStateSetKeyReleased(input, KR_OTHER);
         return 1;
     }
 
-    INPUT_RETURN_VALUE retVal = ProcessKey(ipc->owner, type,
+    INPUT_RETURN_VALUE retVal = FcitxInstanceProcessKey(ipc->owner, type,
                                            t,
                                            sym, state);
+    
+    if (retVal & IRV_FLAG_ASYNC) {
+        FcitxIPCKeyEvent* event = fcitx_utils_malloc0(sizeof(FcitxIPCKeyEvent));
+        event->sym = originsym;
+        event->state = originstate;
+        FcitxInstancePushKeyEvent(ipc->owner, ipc->frontendid, event);
+    }
 
     if (retVal & IRV_FLAG_FORWARD_KEY || retVal == IRV_TO_PROCESS)
         return 0;
@@ -613,14 +635,14 @@ static void IPCICFocusIn(FcitxIPCFrontend* ipc, FcitxInputContext* ic)
     if (ic == NULL)
         return;
 
-    if (!SetCurrentIC(ipc->owner, ic))
+    if (!FcitxInstanceSetCurrentIC(ipc->owner, ic))
         return;
 
     if (ic) {
-        OnInputFocus(ipc->owner);
+        FcitxUIOnInputFocus(ipc->owner);
     } else {
-        CloseInputWindow(ipc->owner);
-        MoveInputWindow(ipc->owner);
+        FcitxUICloseInputWindow(ipc->owner);
+        FcitxUIMoveInputWindow(ipc->owner);
     }
 
     return;
@@ -628,11 +650,11 @@ static void IPCICFocusIn(FcitxIPCFrontend* ipc, FcitxInputContext* ic)
 
 static void IPCICFocusOut(FcitxIPCFrontend* ipc, FcitxInputContext* ic)
 {
-    FcitxInputContext* currentic = GetCurrentIC(ipc->owner);
+    FcitxInputContext* currentic = FcitxInstanceGetCurrentIC(ipc->owner);
     if (ic && ic == currentic) {
-        SetCurrentIC(ipc->owner, NULL);
-        CloseInputWindow(ipc->owner);
-        OnInputUnFocus(ipc->owner);
+        FcitxInstanceSetCurrentIC(ipc->owner, NULL);
+        FcitxUICloseInputWindow(ipc->owner);
+        FcitxUIOnInputUnFocus(ipc->owner);
     }
 
     return;
@@ -640,10 +662,10 @@ static void IPCICFocusOut(FcitxIPCFrontend* ipc, FcitxInputContext* ic)
 
 static void IPCICReset(FcitxIPCFrontend* ipc, FcitxInputContext* ic)
 {
-    FcitxInputContext* currentic = GetCurrentIC(ipc->owner);
+    FcitxInputContext* currentic = FcitxInstanceGetCurrentIC(ipc->owner);
     if (ic && ic == currentic) {
-        CloseInputWindow(ipc->owner);
-        ResetInput(ipc->owner);
+        FcitxUICloseInputWindow(ipc->owner);
+        FcitxInstanceResetInput(ipc->owner);
     }
 
     return;
@@ -653,7 +675,7 @@ static void IPCICSetCursorLocation(FcitxIPCFrontend* ipc, FcitxInputContext* ic,
 {
     ic->offset_x = x;
     ic->offset_y = y;
-    MoveInputWindow(ipc->owner);
+    FcitxUIMoveInputWindow(ipc->owner);
 
     return;
 }
@@ -667,8 +689,8 @@ void IPCUpdatePreedit(void* arg, FcitxInputContext* ic)
                        FCITX_IC_DBUS_INTERFACE, // interface name of the signal
                        "UpdatePreedit"); // name of the signal
 
-    char* strPreedit = MessagesToCString(FcitxInputStateGetClientPreedit(input));
-    char* str = ProcessOutputFilter(ipc->owner, strPreedit);
+    char* strPreedit = FcitxUIMessagesToCString(FcitxInputStateGetClientPreedit(input));
+    char* str = FcitxInstanceProcessOutputFilter(ipc->owner, strPreedit);
     if (str) {
         free(strPreedit);
         strPreedit = str;
@@ -696,31 +718,31 @@ void IPCUpdateClientSideUI(void* arg, FcitxInputContext* ic)
                        "UpdateClientSideUI"); // name of the signal
 
     char *str;
-    char* strAuxUp = MessagesToCString(FcitxInputStateGetAuxUp(input));
-    str = ProcessOutputFilter(ipc->owner, strAuxUp);
+    char* strAuxUp = FcitxUIMessagesToCString(FcitxInputStateGetAuxUp(input));
+    str = FcitxInstanceProcessOutputFilter(ipc->owner, strAuxUp);
     if (str) {
         free(strAuxUp);
         strAuxUp = str;
     }
-    char* strAuxDown = MessagesToCString(FcitxInputStateGetAuxDown(input));
-    str = ProcessOutputFilter(ipc->owner, strAuxDown);
+    char* strAuxDown = FcitxUIMessagesToCString(FcitxInputStateGetAuxDown(input));
+    str = FcitxInstanceProcessOutputFilter(ipc->owner, strAuxDown);
     if (str) {
         free(strAuxDown);
         strAuxDown = str;
     }
-    char* strPreedit = MessagesToCString(FcitxInputStateGetPreedit(input));
-    str = ProcessOutputFilter(ipc->owner, strPreedit);
+    char* strPreedit = FcitxUIMessagesToCString(FcitxInputStateGetPreedit(input));
+    str = FcitxInstanceProcessOutputFilter(ipc->owner, strPreedit);
     if (str) {
         free(strPreedit);
         strPreedit = str;
     }
-    char* candidateword = CandidateWordToCString(ipc->owner);
-    str = ProcessOutputFilter(ipc->owner, candidateword);
+    char* candidateword = FcitxUICandidateWordToCString(ipc->owner);
+    str = FcitxInstanceProcessOutputFilter(ipc->owner, candidateword);
     if (str) {
         free(candidateword);
         candidateword = str;
     }
-    FcitxIM* im = GetCurrentIM(ipc->owner);
+    FcitxIM* im = FcitxInstanceGetCurrentIM(ipc->owner);
     char* imname = NULL;
     if (im == NULL)
         imname = "En";
@@ -778,7 +800,7 @@ void FcitxDBusPropertyGet(FcitxIPCFrontend* ipc, DBusMessage* message)
         }
         DBusMessageIter args, variant;
         dbus_message_iter_init_append(reply, &args);
-        dbus_message_iter_open_container(&args, DBUS_TYPE_VARIANT, "a(sssb)", &variant );
+        dbus_message_iter_open_container(&args, DBUS_TYPE_VARIANT, "a(sssb)", &variant);
         if (propertTable[index].getfunc)
             propertTable[index].getfunc(ipc, &variant);
         dbus_message_iter_close_container(&args, &variant);
@@ -794,25 +816,25 @@ void FcitxDBusPropertySet(FcitxIPCFrontend* ipc, DBusMessage* message)
     char *interface;
     char *property;
     DBusMessage* reply = dbus_message_new_method_return(message);
-    
+
     DBusMessageIter args, variant;
     dbus_message_iter_init(message, &args);
-    
+
     if (dbus_message_iter_get_arg_type(&args) != DBUS_TYPE_STRING)
         goto dbus_property_set_end;
-    
+
     dbus_message_iter_get_basic(&args, &interface);
     dbus_message_iter_next(&args);
-    
+
     if (dbus_message_iter_get_arg_type(&args) != DBUS_TYPE_STRING)
         goto dbus_property_set_end;
     dbus_message_iter_get_basic(&args, &property);
     dbus_message_iter_next(&args);
-    
+
     if (dbus_message_iter_get_arg_type(&args) != DBUS_TYPE_VARIANT)
         goto dbus_property_set_end;
-    
-    dbus_message_iter_recurse  (&args, &variant);
+
+    dbus_message_iter_recurse(&args, &variant);
     int index = 0;
     while (propertTable[index].interface != NULL) {
         if (strcmp(propertTable[index].interface, interface) == 0
@@ -841,17 +863,16 @@ void FcitxDBusPropertyGetAll(FcitxIPCFrontend* ipc, DBusMessage* message)
         DBusMessageIter args;
         dbus_message_iter_init_append(reply, &args);
         DBusMessageIter array, entry;
-        dbus_message_iter_open_container(&args,DBUS_TYPE_ARRAY,"{sv}", &array);
-    
+        dbus_message_iter_open_container(&args, DBUS_TYPE_ARRAY, "{sv}", &array);
+
         while (propertTable[index].interface != NULL) {
-            if (strcmp(propertTable[index].interface, interface) == 0 && propertTable[index].getfunc)
-            {
+            if (strcmp(propertTable[index].interface, interface) == 0 && propertTable[index].getfunc) {
                 dbus_message_iter_open_container(&array, DBUS_TYPE_DICT_ENTRY,
-                                             NULL, &entry);
-                
+                                                 NULL, &entry);
+
                 dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &propertTable[index].name);
                 DBusMessageIter variant;
-                dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT, propertTable[index].type, &variant );
+                dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT, propertTable[index].type, &variant);
                 propertTable[index].getfunc(ipc, &variant);
                 dbus_message_iter_close_container(&entry, &variant);
 
@@ -893,7 +914,7 @@ void IPCGetPropertyIMList(void* arg, DBusMessageIter* args)
     for (ime = (FcitxIM*) utarray_front(availimes);
             ime != NULL;
             ime = (FcitxIM*) utarray_next(availimes, ime)) {
-        if (!GetIMFromIMList(imes, ime->uniqueName)) {
+        if (!FcitxInstanceGetIMFromIMList(instance, IMAS_Enable, ime->uniqueName)) {
             dbus_message_iter_open_container(&sub, DBUS_TYPE_STRUCT, 0, &ssub);
             boolean enable = false;
             char* name = ime->strName;
@@ -914,15 +935,14 @@ void IPCSetPropertyIMList(void* arg, DBusMessageIter* args)
     FcitxIPCFrontend* ipc = (FcitxIPCFrontend*) arg;
     FcitxInstance* instance = ipc->owner;
     DBusMessageIter sub, ssub;
-    
+
     if (dbus_message_iter_get_arg_type(args) != DBUS_TYPE_ARRAY)
         return;
-    
+
     dbus_message_iter_recurse(args, &sub);
-    
+
     char* result = NULL;
-    while (dbus_message_iter_get_arg_type(&sub) == DBUS_TYPE_STRUCT)
-    {
+    while (dbus_message_iter_get_arg_type(&sub) == DBUS_TYPE_STRUCT) {
         dbus_message_iter_recurse(&sub, &ssub);
         char* name, *uniqueName, *langCode;
         boolean enable;
@@ -930,42 +950,41 @@ void IPCSetPropertyIMList(void* arg, DBusMessageIter* args)
             goto ipc_set_imlist_end;
         dbus_message_iter_get_basic(&ssub, &name);
         dbus_message_iter_next(&ssub);
-        
+
         if (dbus_message_iter_get_arg_type(&ssub) != DBUS_TYPE_STRING)
             goto ipc_set_imlist_end;
         dbus_message_iter_get_basic(&ssub, &uniqueName);
         dbus_message_iter_next(&ssub);
-        
+
         if (dbus_message_iter_get_arg_type(&ssub) != DBUS_TYPE_STRING)
             goto ipc_set_imlist_end;
         dbus_message_iter_get_basic(&ssub, &langCode);
         dbus_message_iter_next(&ssub);
-        
+
         if (dbus_message_iter_get_arg_type(&ssub) != DBUS_TYPE_BOOLEAN)
             goto ipc_set_imlist_end;
         dbus_message_iter_get_basic(&ssub, &enable);
         dbus_message_iter_next(&ssub);
-        
+
         char* newresult;
         if (result == NULL)
-            asprintf(&newresult, "%s:%s", uniqueName, enable ? "True": "False");
+            asprintf(&newresult, "%s:%s", uniqueName, enable ? "True" : "False");
         else
-            asprintf(&newresult, "%s,%s:%s", result, uniqueName, enable ? "True": "False");
+            asprintf(&newresult, "%s,%s:%s", result, uniqueName, enable ? "True" : "False");
         if (result)
             free(result);
         result = newresult;
-ipc_set_imlist_end:
+    ipc_set_imlist_end:
         dbus_message_iter_next(&sub);
     }
-    
+
     FcitxLog(DEBUG, "%s", result);
-    if (result)
-    {
+    if (result) {
         FcitxProfile* profile = FcitxInstanceGetProfile(instance);
         if (profile->imList)
             free(profile->imList);
         profile->imList = result;
-        UpdateIMList(instance);
+        FcitxInstanceUpdateIMList(instance);
     }
 }
 
@@ -973,9 +992,9 @@ void IPCUpdateIMList(void* arg)
 {
     FcitxIPCFrontend* ipc = (FcitxIPCFrontend*) arg;
     DBusMessage* msg = dbus_message_new_signal(FCITX_IM_DBUS_PATH, // object name of the signal
-                    DBUS_INTERFACE_PROPERTIES, // interface name of the signal
-                    "PropertiesChanged"); // name of the signal
-    
+                       DBUS_INTERFACE_PROPERTIES, // interface name of the signal
+                       "PropertiesChanged"); // name of the signal
+
     dbus_uint32_t serial;
     DBusMessageIter args;
     DBusMessageIter changed_properties, invalidated_properties;
@@ -985,14 +1004,14 @@ void IPCUpdateIMList(void* arg)
     char* property = sproperty;
     dbus_message_iter_init_append(msg, &args);
     dbus_message_iter_append_basic(&args, DBUS_TYPE_STRING, &interface);
-    
+
     dbus_message_iter_open_container(&args, DBUS_TYPE_ARRAY, "{sv}", &changed_properties);
     dbus_message_iter_close_container(&args, &changed_properties);
-    
+
     dbus_message_iter_open_container(&args, DBUS_TYPE_ARRAY, "s", &invalidated_properties);
     dbus_message_iter_append_basic(&invalidated_properties, DBUS_TYPE_STRING, &property);
     dbus_message_iter_close_container(&args, &invalidated_properties);
-    
+
     if (!dbus_connection_send(ipc->conn, msg, &serial)) {
         FcitxLog(DEBUG, "Out Of Memory!");
     }
