@@ -1,3 +1,22 @@
+/***************************************************************************
+ *   Copyright (C) 2011~2011 by CSSlayer                                   *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU General Public License as published by  *
+ *   the Free Software Foundation; either version 2 of the License, or     *
+ *   (at your option) any later version.                                   *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU General Public License     *
+ *   along with this program; if not, write to the                         *
+ *   Free Software Foundation, Inc.,                                       *
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
+ ***************************************************************************/
+
 #ifndef __FCITX_INPUT_CONTEXT_H_
 #define __FCITX_INPUT_CONTEXT_H_
 
@@ -69,7 +88,7 @@ private:
     QString m_path;
     bool m_enable;
     bool m_has_focus;
-    HOTKEYS m_triggerKey[2];
+    FcitxHotkey m_triggerKey[2];
     QInputContext* m_slave;
     uint m_compose_buffer[FCITX_MAX_COMPOSE_LEN + 1];
     int m_n_compose;
