@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 
 #include "fcitx/fcitx.h"
@@ -1216,7 +1216,7 @@ void InitMHPY(MHPY** pMHPY, const MHPY_TEMPLATE* MHPYtemplate)
 
     iBaseCount++;
 
-    *pMHPY = fcitx_malloc0(sizeof(MHPY) * iBaseCount);
+    *pMHPY = fcitx_utils_malloc0(sizeof(MHPY) * iBaseCount);
 
     MHPY *mhpy = *pMHPY;
 
@@ -1239,7 +1239,7 @@ void InitPYTable(FcitxPinyinConfig* pyconfig)
 
     iBaseCount++;
 
-    pyconfig->PYTable = fcitx_malloc0(sizeof(PYTABLE) * iBaseCount);
+    pyconfig->PYTable = fcitx_utils_malloc0(sizeof(PYTABLE) * iBaseCount);
 
     iBaseCount = 0;
 
