@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 
 #ifndef PYCONFIG_H
@@ -43,7 +43,7 @@ typedef enum _SHUANGPINSCHEME {
 } SHUANGPINSCHEME;
 
 typedef struct _FcitxPinyinConfig {
-    GenericConfig gconfig;
+    FcitxGenericConfig gconfig;
     /* py config */
     int iPinyinPriority;
     int iShuangpinPriority;
@@ -55,9 +55,9 @@ typedef struct _FcitxPinyinConfig {
     ADJUSTORDER baseOrder;
     ADJUSTORDER phraseOrder;
     ADJUSTORDER freqOrder;
-    HOTKEYS hkPYAddFreq[2];
-    HOTKEYS hkPYDelFreq[2];
-    HOTKEYS hkPYDelUserPhr[2];
+    FcitxHotkey hkPYAddFreq[2];
+    FcitxHotkey hkPYDelFreq[2];
+    FcitxHotkey hkPYDelUserPhr[2];
     char* strPYGetWordFromPhrase;
     char cPYYCDZ[3];
     struct _MHPY *MHPY_C;
