@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 
 /**
@@ -33,8 +33,10 @@
 #include <stdio.h>
 #include <sys/stat.h>
 #include <libgen.h>
+#include <dirent.h>
 
 #include "fcitx/fcitx.h"
+#include "fcitx-utils/utils.h"
 #include "xdg.h"
 
 static void make_path(const char *path);
@@ -42,12 +44,11 @@ static void make_path(const char *path);
 void
 make_path(const char *path)
 {
-    char opath[PATH_MAX];
+    char *opath;
     char *p;
     size_t len;
 
-    strncpy(opath, path, sizeof(opath));
-    opath[PATH_MAX - 1] = '\0';
+    opath = strdup(path);
     len = strlen(opath);
 
     while (opath[len - 1] == '/') {
@@ -67,58 +68,62 @@ make_path(const char *path)
 
     if (access(opath, F_OK))        /* if path is not terminated with / */
         mkdir(opath, S_IRWXU);
+
+    free(opath);
 }
 
 
 FCITX_EXPORT_API
-FILE *GetXDGFileWithPrefix(const char* prefix, const char *fileName, const char *mode, char **retFile)
+FILE *FcitxXDGGetFileWithPrefix(const char* prefix, const char *fileName, const char *mode, char **retFile)
 {
     size_t len;
-    char prefixpath[PATH_MAX];
-    snprintf(prefixpath, PATH_MAX, "%s/%s", PACKAGE, prefix);
-    char ** path = GetXDGPath(&len, "XDG_CONFIG_HOME", ".config", prefixpath , DATADIR, prefixpath);
+    char *prefixpath;
+    asprintf(&prefixpath, "%s/%s", PACKAGE, prefix);
+    char ** path = FcitxXDGGetPath(&len, "XDG_CONFIG_HOME", ".config", prefixpath , DATADIR, prefixpath);
+    free(prefixpath);
 
-    FILE* fp = GetXDGFile(fileName, path, mode, len, retFile);
+    FILE* fp = FcitxXDGGetFile(fileName, path, mode, len, retFile);
 
-    FreeXDGPath(path);
+    FcitxXDGFreePath(path);
 
     return fp;
 }
 
 FCITX_EXPORT_API
-FILE *GetLibFile(const char *filename, const char *mode, char **retFile)
+FILE *FcitxXDGGetLibFile(const char *filename, const char *mode, char **retFile)
 {
     size_t len;
     char ** path;
-    path = GetXDGPath(&len, "XDG_CONFIG_HOME", ".config", PACKAGE "/lib" , LIBDIR, PACKAGE);
+    path = FcitxXDGGetPath(&len, "XDG_CONFIG_HOME", ".config", PACKAGE "/lib" , LIBDIR, PACKAGE);
 
-    FILE* fp = GetXDGFile(filename, path, mode, len, retFile);
+    FILE* fp = FcitxXDGGetFile(filename, path, mode, len, retFile);
 
-    FreeXDGPath(path);
+    FcitxXDGFreePath(path);
 
     return fp;
 
 }
 
 FCITX_EXPORT_API
-FILE *GetXDGFileUserWithPrefix(const char* prefix, const char *fileName, const char *mode, char **retFile)
+FILE *FcitxXDGGetFileUserWithPrefix(const char* prefix, const char *fileName, const char *mode, char **retFile)
 {
     size_t len;
-    char prefixpath[PATH_MAX];
-    snprintf(prefixpath, PATH_MAX, "%s/%s", PACKAGE, prefix);
-    char ** path = GetXDGPath(&len, "XDG_CONFIG_HOME", ".config", prefixpath , NULL, NULL);
+    char *prefixpath;
+    asprintf(&prefixpath, "%s/%s", PACKAGE, prefix);
+    char ** path = FcitxXDGGetPath(&len, "XDG_CONFIG_HOME", ".config", prefixpath , NULL, NULL);
+    free(prefixpath);
 
-    FILE* fp = GetXDGFile(fileName, path, mode, len, retFile);
+    FILE* fp = FcitxXDGGetFile(fileName, path, mode, len, retFile);
 
-    FreeXDGPath(path);
+    FcitxXDGFreePath(path);
 
     return fp;
 }
 
 FCITX_EXPORT_API
-FILE *GetXDGFile(const char *fileName, char **path, const char *mode, size_t len, char **retFile)
+FILE *FcitxXDGGetFile(const char *fileName, char **path, const char *mode, size_t len, char **retFile)
 {
-    char buf[PATH_MAX];
+    char* buf = NULL;
     size_t i;
     FILE *fp = NULL;
 
@@ -136,31 +141,27 @@ FILE *GetXDGFile(const char *fileName, char **path, const char *mode, size_t len
     if (len <= 0)
         return NULL;
 
-    if (!mode) {
-        snprintf(buf, sizeof(buf), "%s/%s", path[0], fileName);
-        buf[sizeof(buf) - 1] = '\0';
-
-        if (retFile)
-            *retFile = strdup(buf);
+    if (!mode && retFile) {
+        asprintf(retFile, "%s/%s", path[0], fileName);
 
         return NULL;
     }
 
     for (i = 0; i < len; i++) {
-        snprintf(buf, sizeof(buf), "%s/%s", path[i], fileName);
-        buf[sizeof(buf) - 1] = '\0';
+        asprintf(&buf, "%s/%s", path[i], fileName);
 
         fp = fopen(buf, mode);
 
         if (fp)
             break;
 
+        free(buf);
+        buf = NULL;
     }
 
     if (!fp) {
         if (strchr(mode, 'w') || strchr(mode, 'a')) {
-            snprintf(buf, sizeof(buf), "%s/%s", path[0], fileName);
-            buf[sizeof(buf) - 1] = '\0';
+            asprintf(&buf, "%s/%s", path[0], fileName);
 
             char *dirc = strdup(buf);
             char *dir = dirname(dirc);
@@ -171,20 +172,22 @@ FILE *GetXDGFile(const char *fileName, char **path, const char *mode, size_t len
     }
 
     if (retFile)
-        *retFile = strdup(buf);
+        *retFile = buf;
+    else if (buf)
+        free(buf);
 
     return fp;
 }
 
 FCITX_EXPORT_API
-void FreeXDGPath(char **path)
+void FcitxXDGFreePath(char **path)
 {
     free(path[0]);
     free(path);
 }
 
 FCITX_EXPORT_API
-char **GetXDGPath(
+char **FcitxXDGGetPath(
     size_t *len,
     const char* homeEnv,
     const char* homeDefault,
@@ -246,4 +249,78 @@ char **GetXDGPath(
     return dirsArray;
 }
 
+FCITX_EXPORT_API
+FcitxStringHashSet* FcitxXDGGetFiles(
+    char* path,
+    char* prefix,
+    char* suffix
+)
+{
+    char **xdgPath;
+    size_t len;
+    char *pathBuf;
+    size_t i = 0;
+    DIR *dir;
+    struct dirent *drt;
+    struct stat fileStat;
+
+    FcitxStringHashSet* sset = NULL;
+
+    char *prefixpath;
+    asprintf(&prefixpath, "%s/%s", PACKAGE, path);
+    xdgPath = FcitxXDGGetPath(&len, "XDG_CONFIG_HOME", ".config" , prefixpath , DATADIR , prefixpath);
+    free(prefixpath);
+
+    for (i = 0; i < len; i++) {
+        asprintf(&pathBuf, "%s", xdgPath[i]);
+
+        dir = opendir(pathBuf);
+        free(pathBuf);
+        if (dir == NULL)
+            continue;
+        
+        size_t suffixlen = 0;
+        size_t prefixlen = 0;
+        
+        if (suffix) suffixlen = strlen(suffix);
+        if (prefix) prefixlen = strlen(prefix);
+
+        /* collect all *.conf files */
+        while ((drt = readdir(dir)) != NULL) {
+            size_t nameLen = strlen(drt->d_name);
+            if (nameLen <= suffixlen + prefixlen)
+                continue;
+
+            if (suffix && strcmp(drt->d_name + nameLen - suffixlen, suffix) != 0)
+                continue;
+            if (prefix && strncmp(drt->d_name, prefix, prefixlen) != 0)
+                continue;
+            asprintf(&pathBuf, "%s/%s", xdgPath[i], drt->d_name);
+
+            int statresult = stat(pathBuf, &fileStat);
+            free(pathBuf);
+            if (statresult == -1)
+                continue;
+
+            if (fileStat.st_mode & S_IFREG) {
+                FcitxStringHashSet *string;
+                HASH_FIND_STR(sset, drt->d_name, string);
+                if (!string) {
+                    char *bStr = strdup(drt->d_name);
+                    string = malloc(sizeof(FcitxStringHashSet));
+                    memset(string, 0, sizeof(FcitxStringHashSet));
+                    string->name = bStr;
+                    HASH_ADD_KEYPTR(hh, sset, string->name, strlen(string->name), string);
+                }
+            }
+        }
+
+        closedir(dir);
+    }
+
+    FcitxXDGFreePath(xdgPath);
+
+    return sset;
+}
+
 // kate: indent-mode cstyle; space-indent on; indent-width 0;
