@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 
 #ifndef _FCITX_UTF8_H_
@@ -31,14 +31,14 @@ extern "C" {
 
 #define ISUTF8_CB(c)  (((c)&0xc0) == 0x80)
 
-    extern size_t utf8_strlen(const char *s);
-    extern char* utf8_get_char(const char *in, int *chr);
-    extern int utf8_strncmp(const char *s1, const char *s2, int n);
-    extern int utf8_char_len(const char *in);
-    extern char* utf8_get_nth_char(char* s, unsigned int n);
-    extern int utf8_check_string(const char *s);
-    extern int utf8_get_char_extended(const char *p, int max_len);
-    extern int utf8_get_char_validated(const char *p, int max_len);
+    size_t fcitx_utf8_strlen(const char *s);
+    char*  fcitx_utf8_get_char(const char *in, int *chr);
+    int    fcitx_utf8_strncmp(const char *s1, const char *s2, int n);
+    int    fcitx_utf8_char_len(const char *in);
+    char*  fcitx_utf8_get_nth_char(char* s, unsigned int n);
+    int    fcitx_utf8_check_string(const char *s);
+    int    fcitx_utf8_get_char_extended(const char *p, int max_len);
+    int    fcitx_utf8_get_char_validated(const char *p, int max_len);
 
 #ifdef __cplusplus
 }
