@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 /**
  * @file   utils.h
@@ -39,7 +39,7 @@ extern "C" {
     /**
      * @brief A hash set for string
      **/
-    typedef struct _StringHashSet {
+    typedef struct _FcitxStringHashSet {
         /**
          * @brief String in Hash Set
          **/
@@ -48,7 +48,7 @@ extern "C" {
          * @brief UT Hash handle
          **/
         UT_hash_handle hh;
-    } StringHashSet;
+    } FcitxStringHashSet;
 
 
     /**
@@ -63,16 +63,16 @@ extern "C" {
      *
      * @return
      */
-    void *custom_bsearch(const void *key, const void *base,
-                         size_t nmemb, size_t size, int accurate,
-                         int (*compar)(const void *, const void *));
+    void *fcitx_utils_custom_bsearch(const void *key, const void *base,
+                                     size_t nmemb, size_t size, int accurate,
+                                     int (*compar)(const void *, const void *));
 
     /**
      * @brief Fork twice to run as daemon
      *
      * @return void
      **/
-    void InitAsDaemon();
+    void fcitx_utils_init_as_daemon();
 
     /**
      * @brief Count the file line number
@@ -80,7 +80,15 @@ extern "C" {
      * @param fpDict file pointer
      * @return int line number
      **/
-    int CalculateRecordNumber(FILE* fpDict);
+    int fcitx_utils_calculate_record_number(FILE* fpDict);
+    
+    
+    /**
+     * @brief new empty string list
+     *
+     * @return UT_array*
+     **/
+    UT_array* fcitx_utils_new_string_list();
 
     /**
      * @brief Split a string by delm
@@ -89,7 +97,25 @@ extern "C" {
      * @param delm character as delimiter
      * @return UT_array* a new utarray for store the split string
      **/
-    UT_array* SplitString(const char *str, char delm);
+    UT_array* fcitx_utils_split_string(const char *str, char delm);
+    
+    /**
+     * @brief append a string with printf format
+     *
+     * @param list string list
+     * @param fmt printf fmt
+     * @return void
+     **/
+    void fcitx_utils_string_list_printf_append(UT_array* list, const char* fmt,...);
+    
+    /**
+     * @brief Join string list with delm
+     *
+     * @param list string list
+     * @param delm delm
+     * @return char* return string, need to be free'd
+     **/
+    char* fcitx_utils_join_string_list(UT_array* list, char delm);
 
     /**
      * @brief Helper function for free the SplitString Output
@@ -98,7 +124,17 @@ extern "C" {
      * @return void
      * @see SplitString
      **/
-    void FreeStringList(UT_array *list);
+    void fcitx_utils_free_string_list(UT_array *list);
+
+    /**
+     * @brief Free String Hash Set
+     *
+     * @param sset String Hash Set
+     * @return void
+     *
+     * @since 4.1.3
+     **/
+    void fcitx_utils_free_string_hash_set(FcitxStringHashSet* sset);
 
     /**
      * @brief Trim the input string's white space
@@ -106,7 +142,7 @@ extern "C" {
      * @param s input string
      * @return char* new malloced string, need to free'd by caller
      **/
-    char* fcitx_trim(char *s);
+    char* fcitx_utils_trim(const char *s);
 
     /**
      * @brief Malloc and memset all memory to zero
@@ -114,11 +150,28 @@ extern "C" {
      * @param bytes malloc size
      * @return void* malloced pointer
      **/
-    void* fcitx_malloc0(size_t bytes);
+    void* fcitx_utils_malloc0(size_t bytes);
 
-    int FcitxGetDisplayNumber();
+    /**
+     * @brief Get Display number
+     *
+     * @return int
+     **/
+    int fcitx_utils_get_display_number();
+    
+    /**
+     * @brief Get current language code, result need to be free'd
+     *
+     * @return char*
+     **/
+    char* fcitx_utils_get_current_langcode();
 
-    char* fcitx_get_process_name();
+    /**
+     * @brief Get Current Process Name
+     *
+     * @return char*
+     **/
+    char* fcitx_utils_get_process_name();
 
 #ifdef __cplusplus
 }
