@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 
 /**
@@ -24,8 +24,6 @@
  * @author CSSlayer wengxt@gmail.com
  */
 
-#include <limits.h>
-#include <dirent.h>
 #include <sys/stat.h>
 #include <libintl.h>
 #include <dlfcn.h>
@@ -50,10 +48,13 @@ CONFIG_BINDING_REGISTER("Addon", "Type", type)
 CONFIG_BINDING_REGISTER("Addon", "Dependency", depend)
 CONFIG_BINDING_REGISTER("Addon", "Priority", priority)
 CONFIG_BINDING_REGISTER("Addon", "SubConfig", subconfig)
+CONFIG_BINDING_REGISTER("Addon", "IMRegisterMethod", registerMethod)
+CONFIG_BINDING_REGISTER("Addon", "IMRegisterArgument", registerArgument)
+CONFIG_BINDING_REGISTER("Addon", "UIFallback", uifallback)
 CONFIG_BINDING_END()
 
 static const UT_icd function_icd = {sizeof(void*), 0, 0 , 0};
-static const UT_icd addon_icd = {sizeof(FcitxAddon), NULL , NULL, FreeAddon};
+static const UT_icd addon_icd = {sizeof(FcitxAddon), NULL , NULL, FcitxAddonFree};
 static int AddonPriorityCmp(const void* a, const void* b)
 {
     FcitxAddon *aa = (FcitxAddon*)a, *ab = (FcitxAddon*)b;
@@ -61,7 +62,7 @@ static int AddonPriorityCmp(const void* a, const void* b)
 }
 
 FCITX_EXPORT_API
-void InitFcitxAddons(UT_array* addons)
+void FcitxAddonsInit(UT_array* addons)
 {
     utarray_init(addons, &addon_icd);
 }
@@ -70,108 +71,63 @@ void InitFcitxAddons(UT_array* addons)
  * @brief Load Addon Info
  */
 FCITX_EXPORT_API
-void LoadAddonInfo(UT_array* addons)
+void FcitxAddonsLoad(UT_array* addons)
 {
     char **addonPath;
     size_t len;
-    char pathBuf[PATH_MAX];
     size_t i = 0;
-    DIR *dir;
-    struct dirent *drt;
-    struct stat fileStat;
     utarray_clear(addons);
 
-    StringHashSet* sset = NULL;
-
-    addonPath = GetXDGPath(&len, "XDG_CONFIG_HOME", ".config", PACKAGE "/addon" , DATADIR, PACKAGE "/addon");
-
-    for (i = 0; i < len; i++) {
-        snprintf(pathBuf, sizeof(pathBuf), "%s", addonPath[i]);
-        pathBuf[sizeof(pathBuf) - 1] = '\0';
-
-        dir = opendir(pathBuf);
-        if (dir == NULL)
-            continue;
-
-        /* collect all *.conf files */
-        while ((drt = readdir(dir)) != NULL) {
-            size_t nameLen = strlen(drt->d_name);
-            if (nameLen <= strlen(".conf"))
-                continue;
-            memset(pathBuf, 0, sizeof(pathBuf));
-
-            if (strcmp(drt->d_name + nameLen - strlen(".conf"), ".conf") != 0)
-                continue;
-            snprintf(pathBuf, sizeof(pathBuf), "%s/%s", addonPath[i], drt->d_name);
-
-            if (stat(pathBuf, &fileStat) == -1)
-                continue;
-
-            if (fileStat.st_mode & S_IFREG) {
-                StringHashSet *string;
-                HASH_FIND_STR(sset, drt->d_name, string);
-                if (!string) {
-                    char *bStr = strdup(drt->d_name);
-                    string = malloc(sizeof(StringHashSet));
-                    memset(string, 0, sizeof(StringHashSet));
-                    string->name = bStr;
-                    HASH_ADD_KEYPTR(hh, sset, string->name, strlen(string->name), string);
-                }
-            }
-        }
-
-        closedir(dir);
-    }
-
+    FcitxStringHashSet* sset = FcitxXDGGetFiles(
+                              "addon",
+                              NULL,
+                              ".conf"
+                          );
+    addonPath = FcitxXDGGetPath(&len, "XDG_CONFIG_HOME", ".config", PACKAGE "/addon" , DATADIR, PACKAGE "/addon");
     char **paths = malloc(sizeof(char*) * len);
     for (i = 0; i < len ; i ++)
-        paths[i] = malloc(sizeof(char) * PATH_MAX);
-    StringHashSet* string;
+        paths[i] = NULL;
+    FcitxStringHashSet* string;
     for (string = sset;
             string != NULL;
-            string = (StringHashSet*)string->hh.next) {
+            string = (FcitxStringHashSet*)string->hh.next) {
         int i = 0;
         for (i = len - 1; i >= 0; i--) {
-            snprintf(paths[i], PATH_MAX, "%s/%s", addonPath[len - i - 1], string->name);
+            asprintf(&paths[i], "%s/%s", addonPath[len - i - 1], string->name);
             FcitxLog(DEBUG, "Load Addon Config File:%s", paths[i]);
         }
         FcitxLog(INFO, _("Load Addon Config File:%s"), string->name);
-        ConfigFile* cfile = ParseMultiConfigFile(paths, len, GetAddonConfigDesc());
+        FcitxConfigFile* cfile = FcitxConfigParseMultiConfigFile(paths, len, FcitxAddonGetConfigDesc());
         if (cfile) {
-            FcitxAddon addon;
-            memset(&addon, 0, sizeof(FcitxAddon));
-            utarray_push_back(addons, &addon);
+            utarray_extend_back(addons);
             FcitxAddon *a = (FcitxAddon*) utarray_back(addons);
             utarray_init(&a->functionList, &function_icd);
-            FcitxAddonConfigBind(a, cfile, GetAddonConfigDesc());
-            ConfigBindSync((GenericConfig*)a);
+            FcitxAddonConfigBind(a, cfile, FcitxAddonGetConfigDesc());
+            FcitxConfigBindSync((FcitxGenericConfig*)a);
             FcitxLog(DEBUG, _("Addon Config %s is %s"), string->name, (a->bEnabled) ? "Enabled" : "Disabled");
         }
-    }
 
-    for (i = 0; i < len ; i ++)
-        free(paths[i]);
+        for (i = 0; i < len ; i ++) {
+            free(paths[i]);
+            paths[i] = NULL;
+        }
+    }
     free(paths);
 
-    FreeXDGPath(addonPath);
+    FcitxXDGFreePath(addonPath);
 
-    StringHashSet *curStr;
-    while (sset) {
-        curStr = sset;
-        HASH_DEL(sset, curStr);
-        free(curStr->name);
-        free(curStr);
-    }
+    fcitx_utils_free_string_hash_set(sset);
 
     utarray_sort(addons, AddonPriorityCmp);
 }
 
 FCITX_EXPORT_API
-void AddonResolveDependency(FcitxInstance* instance)
+void FcitxInstanceResolveAddonDependency(FcitxInstance* instance)
 {
     UT_array* addons = &instance->addons;
     boolean remove = true;
     FcitxAddon *addon;
+    FcitxAddon *uiaddon = NULL;
 
     /* choose ui */
     boolean founduiflag = false;
@@ -181,16 +137,38 @@ void AddonResolveDependency(FcitxInstance* instance)
         if (addon->category == AC_UI) {
             if (instance->uiname == NULL) {
                 if (addon->bEnabled) {
-                    if (!founduiflag)
+                    if (!founduiflag) {
+                        uiaddon = addon;
                         founduiflag = true;
-                    else
+                    } else
                         addon->bEnabled = false;
                 }
             } else {
                 if (strcmp(instance->uiname, addon->name) != 0)
                     addon->bEnabled = false;
-                else
+                else {
+                    uiaddon = addon;
                     addon->bEnabled = true;
+                }
+            }
+        }
+    }
+
+    if (uiaddon && uiaddon->uifallback) {
+        for (addon = (FcitxAddon *) utarray_front(addons);
+                addon != NULL;
+                addon = (FcitxAddon *) utarray_next(addons, addon)) {
+            if (addon->category == AC_UI && strcmp(uiaddon->uifallback, addon->name) == 0) {
+                addon->bEnabled = true;
+                FcitxAddon temp;
+                int uiidx = utarray_eltidx(addons, uiaddon);
+                int fallbackidx = utarray_eltidx(addons, addon);
+                if (fallbackidx < uiidx) {
+                    temp = *uiaddon;
+                    *uiaddon = *addon;
+                    *addon = temp;
+                }
+                break;
             }
         }
     }
@@ -202,13 +180,13 @@ void AddonResolveDependency(FcitxInstance* instance)
                 addon = (FcitxAddon *) utarray_next(addons, addon)) {
             if (!addon->bEnabled)
                 continue;
-            UT_array* dependlist = SplitString(addon->depend, ',');
+            UT_array* dependlist = fcitx_utils_split_string(addon->depend, ',');
             boolean valid = true;
             char **depend = NULL;
             for (depend = (char **) utarray_front(dependlist);
                     depend != NULL;
                     depend = (char **) utarray_next(dependlist, depend)) {
-                if (!AddonIsAvailable(addons, *depend)) {
+                if (!FcitxAddonsIsAddonAvailable(addons, *depend)) {
                     valid = false;
                     break;
                 }
@@ -224,7 +202,7 @@ void AddonResolveDependency(FcitxInstance* instance)
 }
 
 FCITX_EXPORT_API
-boolean AddonIsAvailable(UT_array* addons, const char* name)
+boolean FcitxAddonsIsAddonAvailable(UT_array* addons, const char* name)
 {
     FcitxAddon *addon;
     for (addon = (FcitxAddon *) utarray_front(addons);
@@ -237,7 +215,7 @@ boolean AddonIsAvailable(UT_array* addons, const char* name)
 }
 
 FCITX_EXPORT_API
-FcitxAddon* GetAddonByName(UT_array* addons, const char* name)
+FcitxAddon* FcitxAddonsGetAddonByName(UT_array* addons, const char* name)
 {
     FcitxAddon *addon;
     for (addon = (FcitxAddon *) utarray_front(addons);
@@ -255,15 +233,15 @@ FcitxAddon* GetAddonByName(UT_array* addons, const char* name)
  * @return the description of addon configure.
  */
 FCITX_EXPORT_API
-CONFIG_DESC_DEFINE(GetAddonConfigDesc, "addon.desc")
+CONFIG_DESC_DEFINE(FcitxAddonGetConfigDesc, "addon.desc")
 
 FCITX_EXPORT_API
-void FreeAddon(void *v)
+void FcitxAddonFree(void* v)
 {
     FcitxAddon *addon = (FcitxAddon*) v;
     if (!addon)
         return ;
-    FreeConfigFile(addon->config.configFile);
+    FcitxConfigFreeConfigFile(addon->config.configFile);
     free(addon->name);
     free(addon->library);
     free(addon->comment);
