@@ -15,24 +15,25 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 
 #include "candidate.h"
 
-typedef struct _CandidateWordList {
+typedef struct _FcitxCandidateWordList {
     UT_array candWords;
     char strChoose[MAX_CAND_WORD + 1];
+    unsigned int candiateModifier;
     int currentPage;
     int wordPerPage;
-} CandidateWordList;
+} FcitxCandidateWordList;
 
-const UT_icd cand_icd = { sizeof(CandidateWord), NULL, NULL, CandidateWordFree };
+const UT_icd cand_icd = { sizeof(FcitxCandidateWord), NULL, NULL, FcitxCandidateWordFree };
 
 FCITX_EXPORT_API
-CandidateWordList* CandidateWordInit()
+FcitxCandidateWordList* FcitxCandidateWordNewList()
 {
-    CandidateWordList* candList = fcitx_malloc0(sizeof(CandidateWordList));
+    FcitxCandidateWordList* candList = fcitx_utils_malloc0(sizeof(FcitxCandidateWordList));
 
     utarray_init(&candList->candWords, &cand_icd);
     utarray_reserve(&candList->candWords, 128);
@@ -43,29 +44,38 @@ CandidateWordList* CandidateWordInit()
 }
 
 FCITX_EXPORT_API
-void CandidateWordInsert(CandidateWordList* candList, CandidateWord* candWord, int position)
+void FcitxCandidateWordInsert(FcitxCandidateWordList* candList, FcitxCandidateWord* candWord, int position)
 {
     utarray_insert(&candList->candWords, candWord, position);
 }
 
 FCITX_EXPORT_API
-void CandidateWordReset(CandidateWordList* candList)
+void FcitxCandidateWordRemove(FcitxCandidateWordList* candList, FcitxCandidateWord* candWord)
+{
+    int idx = utarray_eltidx(&candList->candWords, candWord);
+    if (idx < 0 || idx >= utarray_len(&candList->candWords))
+        return;
+    utarray_erase(&candList->candWords, idx, 1);
+}
+
+FCITX_EXPORT_API
+void FcitxCandidateWordReset(FcitxCandidateWordList* candList)
 {
     utarray_clear(&candList->candWords);
     candList->currentPage = 0;
 }
 
 FCITX_EXPORT_API
-CandidateWord* CandidateWordGetCurrentWindow(CandidateWordList* candList)
+FcitxCandidateWord* FcitxCandidateWordGetCurrentWindow(FcitxCandidateWordList* candList)
 {
-    return (CandidateWord*) utarray_eltptr(&candList->candWords, candList->currentPage * candList->wordPerPage);
+    return (FcitxCandidateWord*) utarray_eltptr(&candList->candWords, candList->currentPage * candList->wordPerPage);
 }
 
 FCITX_EXPORT_API
-CandidateWord* CandidateWordGetCurrentWindowNext(CandidateWordList* candList, CandidateWord* candWord)
+FcitxCandidateWord* FcitxCandidateWordGetCurrentWindowNext(FcitxCandidateWordList* candList, FcitxCandidateWord* candWord)
 {
-    CandidateWord* nextCandWord = (CandidateWord*) utarray_next(&candList->candWords, candWord);
-    CandidateWord* startCandWord = CandidateWordGetCurrentWindow(candList);
+    FcitxCandidateWord* nextCandWord = (FcitxCandidateWord*) utarray_next(&candList->candWords, candWord);
+    FcitxCandidateWord* startCandWord = FcitxCandidateWordGetCurrentWindow(candList);
     if (nextCandWord == NULL)
         return NULL;
     int delta = utarray_eltidx(&candList->candWords, nextCandWord) - utarray_eltidx(&candList->candWords, startCandWord);
@@ -75,25 +85,28 @@ CandidateWord* CandidateWordGetCurrentWindowNext(CandidateWordList* candList, Ca
 }
 
 FCITX_EXPORT_API
-CandidateWord* CandidateWordGetByIndex(CandidateWordList* candList, int index)
+FcitxCandidateWord* FcitxCandidateWordGetByIndex(FcitxCandidateWordList* candList, int index)
 {
     if (index < candList->wordPerPage && index >= 0)
-        return (CandidateWord*) utarray_eltptr(&candList->candWords, candList->currentPage * candList->wordPerPage + index);
+        return (FcitxCandidateWord*) utarray_eltptr(&candList->candWords, candList->currentPage * candList->wordPerPage + index);
     return NULL;
 }
 
 FCITX_EXPORT_API
-INPUT_RETURN_VALUE CandidateWordChooseByIndex(CandidateWordList* candList, int index)
+INPUT_RETURN_VALUE FcitxCandidateWordChooseByIndex(FcitxCandidateWordList* candList, int index)
 {
-    CandidateWord* candWord = CandidateWordGetByIndex(candList, index);
-    if (candWord == NULL)
-        return IRV_TO_PROCESS;
-    else
+    FcitxCandidateWord* candWord = FcitxCandidateWordGetByIndex(candList, index);
+    if (candWord == NULL) {
+        if (FcitxCandidateWordGetListSize(candList) > 0)
+            return IRV_DO_NOTHING;
+        else
+            return IRV_TO_PROCESS;
+    } else
         return candWord->callback(candWord->owner, candWord);
 }
 
 FCITX_EXPORT_API
-boolean CandidateWordHasPrev(CandidateWordList* candList)
+boolean FcitxCandidateWordHasPrev(FcitxCandidateWordList* candList)
 {
     if (candList->currentPage > 0)
         return true;
@@ -102,24 +115,24 @@ boolean CandidateWordHasPrev(CandidateWordList* candList)
 }
 
 FCITX_EXPORT_API
-boolean CandidateWordHasNext(CandidateWordList* candList)
+boolean FcitxCandidateWordHasNext(FcitxCandidateWordList* candList)
 {
-    if (candList->currentPage + 1 < CandidateWordPageCount(candList))
+    if (candList->currentPage + 1 < FcitxCandidateWordPageCount(candList))
         return true;
     else
         return false;
 }
 
 FCITX_EXPORT_API
-int CandidateWordPageCount(CandidateWordList* candList)
+int FcitxCandidateWordPageCount(FcitxCandidateWordList* candList)
 {
     return (utarray_len(&candList->candWords) + candList->wordPerPage - 1) / candList->wordPerPage;
 }
 
 FCITX_EXPORT_API
-void CandidateWordFree(void* arg)
+void FcitxCandidateWordFree(void* arg)
 {
-    CandidateWord* candWord = (CandidateWord*) arg;
+    FcitxCandidateWord* candWord = (FcitxCandidateWord*) arg;
     if (candWord->strWord)
         free(candWord->strWord);
     if (candWord->strExtra)
@@ -129,74 +142,90 @@ void CandidateWordFree(void* arg)
 }
 
 FCITX_EXPORT_API
-void CandidateWordAppend(CandidateWordList* candList, CandidateWord* candWord)
+void FcitxCandidateWordAppend(FcitxCandidateWordList* candList, FcitxCandidateWord* candWord)
 {
     utarray_push_back(&candList->candWords, candWord);
 }
 
 FCITX_EXPORT_API
-boolean CandidateWordGoPrevPage(CandidateWordList* candList)
+boolean FcitxCandidateWordGoPrevPage(FcitxCandidateWordList* candList)
 {
-    if (!CandidateWordPageCount(candList))
+    if (!FcitxCandidateWordPageCount(candList))
         return false;
-    if (CandidateWordHasPrev(candList))
+    if (FcitxCandidateWordHasPrev(candList))
         candList->currentPage -- ;
     return true;
 }
 
 FCITX_EXPORT_API
-boolean CandidateWordGoNextPage(CandidateWordList* candList)
+boolean FcitxCandidateWordGoNextPage(FcitxCandidateWordList* candList)
 {
-    if (!CandidateWordPageCount(candList))
+    if (!FcitxCandidateWordPageCount(candList))
         return false;
-    if (CandidateWordHasNext(candList))
+    if (FcitxCandidateWordHasNext(candList))
         candList->currentPage ++ ;
     return true;
 }
 
 FCITX_EXPORT_API
-void CandidateWordSetChoose(CandidateWordList* candList, const char* strChoose)
+void FcitxCandidateWordSetChoose(FcitxCandidateWordList* candList, const char* strChoose)
 {
-    strncpy(candList->strChoose, strChoose, MAX_CAND_WORD);
+    FcitxCandidateWordSetChooseAndModifier(candList, strChoose, FcitxKeyState_None);
 }
 
 FCITX_EXPORT_API
-const char* CandidateWordGetChoose(CandidateWordList* candList)
+void FcitxCandidateWordSetChooseAndModifier(FcitxCandidateWordList* candList, const char* strChoose, unsigned int state)
+{
+    strncpy(candList->strChoose, strChoose, MAX_CAND_WORD);
+    candList->candiateModifier = state;
+}
+
+FCITX_EXPORT_API
+unsigned int FcitxCandidateWordGetModifier(FcitxCandidateWordList* candList)
+{
+    return candList->candiateModifier;
+}
+
+FCITX_EXPORT_API
+const char* FcitxCandidateWordGetChoose(FcitxCandidateWordList* candList)
 {
     return candList->strChoose;
 }
 
 FCITX_EXPORT_API
-int CandidateWordGetCurrentPage(CandidateWordList* candList)
+int FcitxCandidateWordGetCurrentPage(FcitxCandidateWordList* candList)
 {
     return candList->currentPage;
 }
 
 FCITX_EXPORT_API
-void CandidateWordResize(CandidateWordList* candList, int length)
+void FcitxCandidateWordResize(FcitxCandidateWordList* candList, int length)
 {
     utarray_resize(&candList->candWords, length);
 }
 
 FCITX_EXPORT_API
-int CandidateWordGetPageSize(CandidateWordList* candList)
+int FcitxCandidateWordGetPageSize(FcitxCandidateWordList* candList)
 {
     return candList->wordPerPage;
 }
 
 FCITX_EXPORT_API
-void CandidateWordSetPageSize(CandidateWordList* candList, int size)
+void FcitxCandidateWordSetPageSize(FcitxCandidateWordList* candList, int size)
 {
+    if (size <= 0 || size > 10)
+        size = 5;
+    
     candList->wordPerPage = size;
 }
 
 FCITX_EXPORT_API
-int CandidateWordGetCurrentWindowSize(CandidateWordList* candList)
+int FcitxCandidateWordGetCurrentWindowSize(FcitxCandidateWordList* candList)
 {
     if (utarray_len(&candList->candWords) == 0)
         return 0;
     /* last page */
-    if (candList->currentPage + 1 == CandidateWordPageCount(candList)) {
+    if (candList->currentPage + 1 == FcitxCandidateWordPageCount(candList)) {
         int size = utarray_len(&candList->candWords) % candList->wordPerPage;
         if (size != 0)
             return size;
@@ -205,20 +234,20 @@ int CandidateWordGetCurrentWindowSize(CandidateWordList* candList)
 }
 
 FCITX_EXPORT_API
-int CandidateWordGetListSize(CandidateWordList* candList)
+int FcitxCandidateWordGetListSize(FcitxCandidateWordList* candList)
 {
     return utarray_len(&candList->candWords);
 }
 
 FCITX_EXPORT_API
-CandidateWord* CandidateWordGetFirst(CandidateWordList* candList)
+FcitxCandidateWord* FcitxCandidateWordGetFirst(FcitxCandidateWordList* candList)
 {
-    return (CandidateWord*) utarray_front(&candList->candWords);
+    return (FcitxCandidateWord*) utarray_front(&candList->candWords);
 }
 
 FCITX_EXPORT_API
-CandidateWord* CandidateWordGetNext(CandidateWordList* candList, CandidateWord* candWord)
+FcitxCandidateWord* FcitxCandidateWordGetNext(FcitxCandidateWordList* candList, FcitxCandidateWord* candWord)
 {
-    return (CandidateWord*) utarray_next(&candList->candWords, candWord);
+    return (FcitxCandidateWord*) utarray_next(&candList->candWords, candWord);
 }
 // kate: indent-mode cstyle; space-indent on; indent-width 0;
