@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 #ifndef __FCITX_INSTANCE_H__
 #define __FCITX_INSTANCE_H__
@@ -34,7 +34,6 @@
 extern "C" {
 #endif
 
-    struct _HookStack;
     struct _FcitxInputContext;
 
     /**
@@ -50,7 +49,23 @@ extern "C" {
      * @param argv argv
      * @return FcitxInstance*
      **/
-    FcitxInstance* CreateFcitxInstance(sem_t *sem, int argc, char* argv[]);
+    FcitxInstance* FcitxInstanceCreate(sem_t *sem, int argc, char* argv[]);
+
+    /**
+     * @brief replace existing fcitx instance
+     *
+     * @param instance fcitx instance
+     * @return boolean
+     **/
+    boolean FcitxInstanceIsTryReplace(FcitxInstance* instance);
+
+    /**
+     * @brief replace existing fcitx instance
+     *
+     * @param instance fcitx instance
+     * @return bool
+     **/
+    void FcitxInstanceResetTryReplace(FcitxInstance* instance);
 
     /**
      * @brief lock the instance
@@ -58,7 +73,7 @@ extern "C" {
      * @param instance fcitx instance
      * @return int
      **/
-    int FcitxLock(FcitxInstance* instance);
+    int FcitxInstanceLock(FcitxInstance* instance);
 
     /**
      * @brief lock the instance
@@ -66,7 +81,7 @@ extern "C" {
      * @param instance fcitx instance
      * @return int
      **/
-    int FcitxUnlock(FcitxInstance* instance);
+    int FcitxInstanceUnlock(FcitxInstance* instance);
 
     /**
      * @brief notify the instance is end
@@ -74,7 +89,7 @@ extern "C" {
      * @param instance fcitx instance
      * @return void
      **/
-    void EndInstance(FcitxInstance* instance);
+    void FcitxInstanceEnd(FcitxInstance* instance);
 
     /**
      * @brief Get Current Input Context
@@ -82,7 +97,7 @@ extern "C" {
      * @param instance
      * @return FcitxInputContext*
      **/
-    FcitxInputContext* GetCurrentIC(struct _FcitxInstance* instance);
+    FcitxInputContext* FcitxInstanceGetCurrentIC(struct _FcitxInstance* instance);
 
     /**
      * @brief Set Current Input Context
@@ -91,8 +106,7 @@ extern "C" {
      * @param ic new input context
      * @return current ic changed
      **/
-    boolean SetCurrentIC(struct _FcitxInstance* instance, FcitxInputContext* ic);
-
+    boolean FcitxInstanceSetCurrentIC(struct _FcitxInstance* instance, FcitxInputContext* ic);
 
     /**
      * @brief Get Addons From Instance
@@ -120,7 +134,7 @@ extern "C" {
 
     void FcitxInstanceSetMaxFD(FcitxInstance* instance, int maxfd);
 
-    FcitxConfig* FcitxInstanceGetConfig(FcitxInstance* instance);
+    FcitxGlobalConfig* FcitxInstanceGetGlobalConfig(FcitxInstance* instance);
 
     FcitxProfile* FcitxInstanceGetProfile(FcitxInstance* instance);
 
