@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 
 #include <stdio.h>
@@ -64,7 +64,7 @@ char *sCornerTrans[] = {
  */
 
 static void* FullWidthCharCreate(FcitxInstance* instance);
-static boolean ProcessFullWidthChar(void* arg, FcitxKeySym sym, unsigned int state, INPUT_RETURN_VALUE* retVal);
+char* ProcessFullWidthChar(void* arg, const char* str);
 static void ToggleFullWidthState(void *arg);
 static boolean GetFullWidthState(void *arg);
 static INPUT_RETURN_VALUE ToggleFullWidthStateWithHotkey(void *arg);
@@ -87,37 +87,58 @@ int ABI_VERSION = FCITX_ABI_VERSION;
 
 void* FullWidthCharCreate(FcitxInstance* instance)
 {
-    FcitxFullWidthChar* fwchar = fcitx_malloc0(sizeof(FcitxFullWidthChar));
-    FcitxConfig* config = FcitxInstanceGetConfig(instance);
+    FcitxFullWidthChar* fwchar = fcitx_utils_malloc0(sizeof(FcitxFullWidthChar));
+    FcitxGlobalConfig* config = FcitxInstanceGetGlobalConfig(instance);
     fwchar->owner = instance;
-    KeyFilterHook hk;
+    FcitxStringFilterHook hk;
     hk.arg = fwchar;
     hk.func = ProcessFullWidthChar;
 
-    RegisterPreInputFilter(instance, hk);
+    FcitxInstanceRegisterCommitFilter(instance, hk);
 
-    HotkeyHook hotkey;
+    FcitxHotkeyHook hotkey;
     hotkey.hotkey = config->hkFullWidthChar;
     hotkey.hotkeyhandle = ToggleFullWidthStateWithHotkey;
     hotkey.arg = fwchar;
 
-    RegisterHotkeyFilter(instance, hotkey);
+    FcitxInstanceRegisterHotkeyFilter(instance, hotkey);
 
-    RegisterStatus(instance, fwchar, "fullwidth", _("Full Width Character"), _("Full Width Character"),  ToggleFullWidthState, GetFullWidthState);
+    FcitxUIRegisterStatus(instance, fwchar, "fullwidth", _("Full Width Character"), _("Full Width Character"),  ToggleFullWidthState, GetFullWidthState);
 
     return fwchar;
 }
 
-boolean ProcessFullWidthChar(void* arg, FcitxKeySym sym, unsigned int state, INPUT_RETURN_VALUE* retVal)
+char* ProcessFullWidthChar(void* arg, const char* str)
 {
     FcitxFullWidthChar* fwchar = (FcitxFullWidthChar*)arg;
     FcitxProfile* profile = FcitxInstanceGetProfile(fwchar->owner);
-    if (profile->bUseFullWidthChar && IsHotKeySimple(sym, state)) {
-        sprintf(GetOutputString(FcitxInstanceGetInputState(fwchar->owner)), "%s", sCornerTrans[sym - 32]);
-        *retVal = IRV_COMMIT_STRING;
-        return true;
+    if (profile->bUseFullWidthChar) {
+        size_t i = 0, ret_len = 0, len = fcitx_utf8_strlen(str);
+        char* ret = (char *) fcitx_utils_malloc0(sizeof(char) * (UTF8_MAX_LENGTH * len + 1));
+        const char* ps = str;
+        ret[0] = '\0';
+        for (; i < len; ++i) {
+            int wc;
+            int chr_len = fcitx_utf8_char_len(ps);
+            char *nps;
+            nps = fcitx_utf8_get_char(ps , &wc);
+
+            if (chr_len == 1 && ps[0] >= '\x20' && ps[0] <= '\x7e') {
+                strcat(ret, sCornerTrans[ps[0] - 32]);
+                ret_len += strlen(sCornerTrans[ps[0] - 32]);
+            } else {
+                strncat(ret, ps, chr_len);
+                ret_len += chr_len;
+            }
+
+            ps = nps;
+
+        }
+        ret[ret_len] = '\0';
+        return ret;
     }
-    return false;
+    else
+        return NULL;
 }
 
 void ToggleFullWidthState(void* arg)
@@ -125,8 +146,8 @@ void ToggleFullWidthState(void* arg)
     FcitxFullWidthChar* fwchar = (FcitxFullWidthChar*)arg;
     FcitxProfile* profile = FcitxInstanceGetProfile(fwchar->owner);
     profile->bUseFullWidthChar = !profile->bUseFullWidthChar;
-    SaveProfile(profile);
-    ResetInput(fwchar->owner);
+    FcitxProfileSave(profile);
+    FcitxInstanceResetInput(fwchar->owner);
 }
 
 boolean GetFullWidthState(void* arg)
@@ -139,7 +160,7 @@ boolean GetFullWidthState(void* arg)
 INPUT_RETURN_VALUE ToggleFullWidthStateWithHotkey(void* arg)
 {
     FcitxFullWidthChar* fwchar = (FcitxFullWidthChar*)arg;
-    UpdateStatus(fwchar->owner, "fullwidth");
+    FcitxUIUpdateStatus(fwchar->owner, "fullwidth");
     return IRV_DO_NOTHING;
 }
 
