@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 #include <limits.h>
 #include <ctype.h>
@@ -34,11 +34,11 @@
 #include "fcitx/frontend.h"
 #include "fcitx-config/xdg.h"
 #include <cairo-xlib.h>
-#include <ui/cairostuff/cairostuff.h>
-#include <ui/classic/classicuiinterface.h>
+#include "ui/cairostuff/cairostuff.h"
+#include "ui/cairostuff/font.h"
+#include "ui/classic/classicuiinterface.h"
 #include "fcitx/hook.h"
-#include <ui/cairostuff/font.h>
-#include <fcitx-utils/utils.h>
+#include "fcitx-utils/utils.h"
 
 #define VK_FILE "vk.conf"
 
@@ -51,12 +51,12 @@ struct _FcitxVKState;
 
 typedef struct _VKS {
     char            strSymbol[VK_NUMBERS][2][UTF8_MAX_LENGTH + 1]; //相应的符号
-    char            strName[MAX_IM_NAME + 1];
+    char           *strName;
 } VKS;
 
 typedef struct _VKWindow {
     Window          window;
-    ConfigColor* fontColor;
+    FcitxConfigColor* fontColor;
     int fontSize;
     cairo_surface_t* surface;
     cairo_surface_t* keyboard;
@@ -115,10 +115,10 @@ static void VKUpdate(void* arg);
 static INPUT_RETURN_VALUE DoVKInput(FcitxVKState* vkstate, KeySym sym, int state);
 static void DisplayVKWindow(VKWindow* vkWindow);
 static boolean VKMenuAction(FcitxUIMenu *menu, int index);
-static void UpdateVKMenuShell(FcitxUIMenu *menu);
+static void UpdateVKMenu(FcitxUIMenu *menu);
 static void SelectVK(FcitxVKState* vkstate, int vkidx);
 
-static ConfigColor blackColor = {0, 0, 0};
+static FcitxConfigColor blackColor = {0, 0, 0};
 
 FCITX_EXPORT_API
 FcitxModule module = {
@@ -134,50 +134,50 @@ int ABI_VERSION = FCITX_ABI_VERSION;
 
 void *VKCreate(FcitxInstance* instance)
 {
-    FcitxVKState *vkstate = fcitx_malloc0(sizeof(FcitxVKState));
-    FcitxConfig* config = FcitxInstanceGetConfig(instance);
+    FcitxVKState *vkstate = fcitx_utils_malloc0(sizeof(FcitxVKState));
+    FcitxGlobalConfig* config = FcitxInstanceGetGlobalConfig(instance);
     vkstate->owner = instance;
-    vkstate->classicui = GetAddonByName(FcitxInstanceGetAddons(instance), FCITX_CLASSIC_UI_NAME);
+    vkstate->classicui = FcitxAddonsGetAddonByName(FcitxInstanceGetAddons(instance), FCITX_CLASSIC_UI_NAME);
 
-    HotkeyHook hotkey;
+    FcitxHotkeyHook hotkey;
     hotkey.hotkey = config->hkVK;
     hotkey.hotkeyhandle = ToggleVKStateWithHotkey;
     hotkey.arg = vkstate;
-    RegisterHotkeyFilter(instance, hotkey);
+    FcitxInstanceRegisterHotkeyFilter(instance, hotkey);
 
-    RegisterStatus(instance, vkstate, "vk", _("Virtual Keyboard"), _("Virtual Keyboard State"),  ToggleVKState, GetVKState);
+    FcitxUIRegisterStatus(instance, vkstate, "vk", _("Virtual Keyboard"), _("Virtual Keyboard State"),  ToggleVKState, GetVKState);
 
     LoadVKMapFile(vkstate);
 
-    KeyFilterHook hk;
+    FcitxKeyFilterHook hk;
     hk.arg = vkstate ;
     hk.func = VKPreFilter;
-    RegisterPreInputFilter(instance, hk);
+    FcitxInstanceRegisterPreInputFilter(instance, hk);
 
     FcitxIMEventHook resethk;
     resethk.arg = vkstate;
     resethk.func = VKReset;
-    RegisterTriggerOnHook(instance, resethk);
-    RegisterTriggerOffHook(instance, resethk);
+    FcitxInstanceRegisterTriggerOnHook(instance, resethk);
+    FcitxInstanceRegisterTriggerOffHook(instance, resethk);
 
     resethk.func = VKUpdate;
-    RegisterInputFocusHook(instance, resethk);
-    RegisterInputUnFocusHook(instance, resethk);
+    FcitxInstanceRegisterInputFocusHook(instance, resethk);
+    FcitxInstanceRegisterInputUnFocusHook(instance, resethk);
 
-    strcpy(vkstate->vkmenu.candStatusBind, "vk");
-    strcpy(vkstate->vkmenu.name, _("Virtual Keyboard"));
+    FcitxMenuInit(&vkstate->vkmenu);
+    vkstate->vkmenu.candStatusBind = strdup("vk");
+    vkstate->vkmenu.name = strdup(_("Virtual Keyboard"));
 
-    utarray_init(&vkstate->vkmenu.shell, &menuICD);
-    vkstate->vkmenu.UpdateMenuShell = UpdateVKMenuShell;
+    vkstate->vkmenu.UpdateMenu = UpdateVKMenu;
     vkstate->vkmenu.MenuAction = VKMenuAction;
     vkstate->vkmenu.priv = vkstate;
     vkstate->vkmenu.isSubMenu = false;
 
     int i;
     for (i = 0; i < vkstate->iVKCount; i ++)
-        AddMenuShell(&vkstate->vkmenu, vkstate->vks[i].strName, MENUTYPE_SIMPLE, NULL);
+        FcitxMenuAddMenuItem(&vkstate->vkmenu, vkstate->vks[i].strName, MENUTYPE_SIMPLE, NULL);
 
-    RegisterMenu(instance, &vkstate->vkmenu);
+    FcitxUIRegisterMenu(instance, &vkstate->vkmenu);
 
     return vkstate;
 }
@@ -189,7 +189,7 @@ boolean VKMenuAction(FcitxUIMenu *menu, int index)
     return true;
 }
 
-void UpdateVKMenuShell(FcitxUIMenu *menu)
+void UpdateVKMenu(FcitxUIMenu *menu)
 {
     FcitxVKState* vkstate = (FcitxVKState*) menu->priv;
     menu->mark = vkstate->iCurrentVK;
@@ -200,7 +200,7 @@ void VKReset(void* arg)
     FcitxVKState *vkstate = (FcitxVKState*) arg;
     VKWindow* vkWindow = vkstate->vkWindow;
     if (vkstate->bVK != false)
-        UpdateStatus(vkstate->owner, "vk");
+        FcitxUIUpdateStatus(vkstate->owner, "vk");
     if (vkWindow)
         XUnmapWindow(vkWindow->dpy, vkWindow->window);
 }
@@ -211,7 +211,7 @@ void VKUpdate(void* arg)
     VKWindow* vkWindow = vkstate->vkWindow;
     if (vkWindow) {
 
-        if (GetCurrentState(vkstate->owner) != IS_CLOSED && vkstate->bVK) {
+        if (FcitxInstanceGetCurrentState(vkstate->owner) != IS_CLOSED && vkstate->bVK) {
             DrawVKWindow(vkWindow);
             DisplayVKWindow(vkWindow);
         } else
@@ -245,7 +245,7 @@ void ToggleVKState(void *arg)
 INPUT_RETURN_VALUE ToggleVKStateWithHotkey(void* arg)
 {
     FcitxVKState *vkstate = (FcitxVKState*) arg;
-    UpdateStatus(vkstate->owner, "vk");
+    FcitxUIUpdateStatus(vkstate->owner, "vk");
     return IRV_DO_NOTHING;
 }
 
@@ -258,7 +258,7 @@ VKWindow* CreateVKWindow(FcitxVKState* vkstate)
     Visual * vs;
     int depth;
     FcitxModuleFunctionArg arg;
-    VKWindow* vkWindow = fcitx_malloc0(sizeof(VKWindow));
+    VKWindow* vkWindow = fcitx_utils_malloc0(sizeof(VKWindow));
     vkWindow->owner = vkstate;
 
     LoadVKImage(vkWindow);
@@ -343,8 +343,7 @@ cairo_surface_t* LoadVKImage(VKWindow* vkWindow)
         return InvokeFunction(vkstate->owner, FCITX_CLASSIC_UI, LOADIMAGE, arg);
     } else {
         if (!vkWindow->keyboard) {
-            char path[PATH_MAX];
-            strncpy(path, PKGDATADIR "/skin/default/keyboard.png" , PATH_MAX);
+            char path[] = PKGDATADIR "/skin/default/keyboard.png";
             vkWindow->keyboard = cairo_image_surface_create_from_png(path);
         }
         return vkWindow->keyboard;
@@ -423,10 +422,10 @@ boolean VKMouseKey(FcitxVKState* vkstate, int x, int y)
     char           *pstr = NULL;
     FcitxInstance* instance = vkstate->owner;
 
-    if (IsInBox(x, y, 1, 1, VK_WINDOW_WIDTH, 16))
+    if (FcitxUIIsInBox(x, y, 1, 1, VK_WINDOW_WIDTH, 16))
         ChangVK(vkstate);
     else {
-        if (GetCurrentIC(instance) == NULL)
+        if (FcitxInstanceGetCurrentIC(instance) == NULL)
             return false;
 
         strKey[1] = '\0';
@@ -437,7 +436,7 @@ boolean VKMouseKey(FcitxVKState* vkstate, int x, int y)
 
             x -= 4;
             if (x >= 313 && x <= 344) { //backspace
-                ForwardKey(instance, GetCurrentIC(instance), FCITX_PRESS_KEY, Key_BackSpace, 0);
+                FcitxInstanceForwardKey(instance, FcitxInstanceGetCurrentIC(instance), FCITX_PRESS_KEY, FcitxKey_BackSpace, 0);
                 return true;
             } else {
                 iIndex = x / 24;
@@ -454,7 +453,7 @@ boolean VKMouseKey(FcitxVKState* vkstate, int x, int y)
                 return false;
 
             if (x >= 4 && x < 38) { //Tab
-                ForwardKey(instance, GetCurrentIC(instance), FCITX_PRESS_KEY, Key_Tab, 0);
+                FcitxInstanceForwardKey(instance, FcitxInstanceGetCurrentIC(instance), FCITX_PRESS_KEY, FcitxKey_Tab, 0);
                 return true;
             } else {
                 iIndex = 13 + (x - 38) / 24;
@@ -503,10 +502,10 @@ boolean VKMouseKey(FcitxVKState* vkstate, int x, int y)
         } else if (y >= 140 && y <= 162) {  //第五行
             if (x >= 4 && x < 38) { //Ins
                 //改变INS键状态
-                ForwardKey(instance, GetCurrentIC(instance), FCITX_PRESS_KEY, Key_Insert, 0);
+                FcitxInstanceForwardKey(instance, FcitxInstanceGetCurrentIC(instance), FCITX_PRESS_KEY, FcitxKey_Insert, 0);
                 return true;
             } else if (x >= 61 && x < 98) { //DEL
-                ForwardKey(instance, GetCurrentIC(instance), FCITX_PRESS_KEY, Key_Delete, 0);
+                FcitxInstanceForwardKey(instance, FcitxInstanceGetCurrentIC(instance), FCITX_PRESS_KEY, FcitxKey_Delete, 0);
                 return true;
             } else if (x >= 99 && x < 270)  //空格
                 strcpy(strKey, " ");
@@ -518,8 +517,8 @@ boolean VKMouseKey(FcitxVKState* vkstate, int x, int y)
         }
 
         if (pstr) {
-            CommitString(instance, GetCurrentIC(instance), pstr);
-            FcitxInstanceIncreateInputCharacterCount(instance, utf8_strlen(pstr));
+            FcitxInstanceCommitString(instance, FcitxInstanceGetCurrentIC(instance), pstr);
+            FcitxInstanceIncreateInputCharacterCount(instance, fcitx_utf8_strlen(pstr));
         }
     }
 
@@ -533,29 +532,31 @@ void LoadVKMapFile(FcitxVKState *vkstate)
 {
     int             i, j;
     FILE           *fp;
-    char            strPath[PATH_MAX];
+    char           *buf = NULL;
     char           *pstr;
     VKS*            vks = vkstate->vks;
+    size_t          len;
 
     for (j = 0; j < VK_MAX; j++) {
         for (i = 0; i < VK_NUMBERS; i++) {
             vks[j].strSymbol[i][0][0] = '\0';
             vks[j].strSymbol[i][1][0] = '\0';
         }
-        vks[j].strName[0] = '\0';
+        if (vks[j].strName) {
+            free(vks[j].strName);
+            vks[j].strName = NULL;
+        }
     }
 
-    fp = GetXDGFileWithPrefix("data", VK_FILE, "rt", NULL);
+    fp = FcitxXDGGetFileWithPrefix("data", VK_FILE, "rt", NULL);
 
     if (!fp)
         return;
 
     vkstate->iVKCount = 0;
 
-    for (;;) {
-        if (!fgets(strPath, PATH_MAX, fp))
-            break;
-        pstr = strPath;
+    while (getline(&buf, &len, fp) != -1) {
+        pstr = buf;
         while (*pstr == ' ' || *pstr == '\t')
             pstr++;
         if (pstr[0] == '#')
@@ -570,7 +571,7 @@ void LoadVKMapFile(FcitxVKState *vkstate)
         if (!strcmp(pstr, "[VK]"))
             vkstate->iVKCount++;
         else if (!strncmp(pstr, "NAME=", 5))
-            strcpy(vks[vkstate->iVKCount - 1].strName, pstr + 5);
+            vks[vkstate->iVKCount - 1].strName = strdup(pstr + 5);
         else {
             if (pstr[1] != '=' && !vkstate->iVKCount)
                 continue;
@@ -604,6 +605,9 @@ void LoadVKMapFile(FcitxVKState *vkstate)
         }
     }
 
+    if (buf)
+        free(buf);
+
     fclose(fp);
 }
 
@@ -674,12 +678,12 @@ INPUT_RETURN_VALUE DoVKInput(FcitxVKState* vkstate, KeySym sym, int state)
     char           *pstr = NULL;
     FcitxInputState *input = FcitxInstanceGetInputState(vkstate->owner);
 
-    if (IsHotKeySimple(sym, state))
+    if (FcitxHotkeyIsHotKeySimple(sym, state))
         pstr = VKGetSymbol(vkstate, sym);
     if (!pstr)
         return IRV_TO_PROCESS;
     else {
-        strcpy(GetOutputString(input), pstr);
+        strcpy(FcitxInputStateGetOutputString(input), pstr);
         return IRV_COMMIT_STRING;
     }
 }
@@ -703,12 +707,12 @@ void SwitchVK(FcitxVKState *vkstate)
         arg.args[1] = &dheight;
         InvokeFunction(vkstate->owner, FCITX_X11, GETSCREENSIZE, arg);
 
-        if (!UISupportMainWindow(instance)) {
+        if (!FcitxUISupportMainWindow(instance)) {
             x = dwidth / 2 - VK_WINDOW_WIDTH / 2;
             y = 0;
         } else {
             int mx = 0, my = 0, mw = 0, mh = 0;
-            GetMainWindowSize(instance, &mx, &my, &mw, &mh);
+            FcitxUIGetMainWindowSize(instance, &mx, &my, &mw, &mh);
             x = mx;
             y = my + mh + 2;
             if ((y + VK_WINDOW_HEIGHT) >= dheight)
@@ -724,17 +728,17 @@ void SwitchVK(FcitxVKState *vkstate)
 
         XMoveWindow(vkWindow->dpy, vkWindow->window, x, y);
         DisplayVKWindow(vkWindow);
-        CloseInputWindow(vkstate->owner);
+        FcitxUICloseInputWindow(instance);
 
-        FcitxInputContext* ic = GetCurrentIC(vkstate->owner);
+        FcitxInputContext* ic = FcitxInstanceGetCurrentIC(instance);
 
-        if (ic && GetCurrentState(instance) == IS_CLOSED)
-            EnableIM(instance, ic, true);
-    } else
+        if (ic && FcitxInstanceGetCurrentState(instance) == IS_CLOSED)
+            FcitxInstanceEnableIM(instance, ic, true);
+    } else {
         XUnmapWindow(vkWindow->dpy, vkWindow->window);
-
-
-
+        FcitxInstanceCleanInputWindow(instance);
+        FcitxUIUpdateInputWindow(instance);
+    }
 }
 
 /*
@@ -744,7 +748,7 @@ void SelectVK(FcitxVKState* vkstate, int vkidx)
 {
     vkstate->bVK = false;
     vkstate->iCurrentVK = vkidx;
-    UpdateStatus(vkstate->owner, "vk");
+    FcitxUIUpdateStatus(vkstate->owner, "vk");
     if (vkstate->vkWindow)
         DrawVKWindow(vkstate->vkWindow);
 }
