@@ -15,7 +15,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.              *
  ***************************************************************************/
 
 #ifndef CLASSICUIINTERFACE_H
@@ -28,7 +28,7 @@
 #define FCITX_CLASSIC_UI_LOADIMAGE 0
 #define FCITX_CLASSIC_UI_LOADIMAGE_RETURNTYPE cairo_surface_t*
 #define FCITX_CLASSIC_UI_GETKEYBOARDFONTCOLOR 1
-#define FCITX_CLASSIC_UI_GETKEYBOARDFONTCOLOR_RETURNTYPE ConfigColor *
+#define FCITX_CLASSIC_UI_GETKEYBOARDFONTCOLOR_RETURNTYPE FcitxConfigColor *
 #define FCITX_CLASSIC_UI_GETFONT 2
 #define FCITX_CLASSIC_UI_GETFONT_RETURNTYPE char**
 
